# -*- coding: utf-8 -*-
# @Time    : 2020/6/11 00:12
# @Author  : CC
# @Desc    : custom_gevent.py
import atexit
import time

from gevent import pool as gevent_pool
from gevent import monkey


class CustomGeventPoolExecutor(gevent_pool.Pool):
    def __init__(self, size=None, greenlet_class=None):
        super().__init__(size, greenlet_class)
        if monkey.is_module_patched('socket') is False:
            raise Exception("""请先打猴子补丁,代码开头加入:
            from gevent import monkey 
            monkey.patch_all()""")
        atexit.register(self.shutdown)

    def submit(self, *args, **kwargs):
        self.spawn(*args, **kwargs)

    def shutdown(self):
        self.join()


if __name__ == '__main__':
    monkey.patch_all()


    def prt(x):
        time.sleep(5)
        print(x)


    pool = CustomGeventPoolExecutor(4)
    for i in range(20):
        pool.submit(prt, i)



# -*- coding: utf-8 -*-
# @Time    : 2020/7/19 13:15
# @Author  : CC
# @Desc    : utils.py
# @Notice  :Please do not use it for commercial use. All consequences are borne by users
import getpass
import hashlib
import socket
import time
import uuid


def str_sha256(text: str) -> str:
    """
    获取字符串hash256值
    :param text: 待hash原始字符串
    :return: hash值
    """
    hash_obj = hashlib.sha256()
    hash_obj.update(text.encode('utf-8'))
    return hash_obj.hexdigest()


def str_sha512(text: str) -> str:
    """
    获取字符串hash512值
    :param text: 待hash原始字符串
    :return: hash值
    """
    hash_obj = hashlib.sha512()
    hash_obj.update(text.encode('utf-8'))
    return hash_obj.hexdigest()


def str_md5(text: str) -> str:
    """
    获取字符串md5值
    :param text: 待hash原始字符串
    :return: hash值
    """
    hash_obj = hashlib.md5()
    hash_obj.update(text.encode('utf-8'))
    return hash_obj.hexdigest()


def gen_uuid():
    return uuid.uuid4().hex


def get_day_str():
    return time.strftime("%Y%m%d%H%M%S", time.localtime())


def sort_dict(dict_obj: dict) -> dict:
    rs = dict()
    if not dict_obj:
        return rs
    for k in sorted(dict_obj.keys()):
        rs[k] = dict_obj[k]
    return rs


def get_now_millseconds():
    return int(time.time() * 1000)


def get_now_seconds():
    return int(time.time())


def get_host_ip():
    """
    查询本机ip地址
    :return: ip
    """
    ip = '127.0.0.1'
    s = socket.socket(type=socket.SOCK_DGRAM)
    try:
        s.connect(('8.8.8.8', 80))
        ip = s.getsockname()[0]
    except Exception:
        pass
    finally:
        s.close()
    return ip


def get_host_name():
    return getpass.getuser()


def get_day_formate(millseconds: float):
    """
    将毫秒转换成日期
    :param millseconds: 毫秒
    :return: 220-04-08 12:34:56
    """
    return time.strftime("%Y-%m-%d %H:%M:%S", time.localtime(millseconds / 1000))


if __name__ == '__main__':
    for st in ['我的地盘', '123', 'abcdefg', '中1abcd']:
        print(type(str_sha256(st)))
        print(str_sha256(st))
    print(get_day_str())
    print(get_host_ip())
    print(get_host_name())
    print(get_now_millseconds())
    print(get_day_formate(get_now_millseconds()))
    print(sort_dict(dict(a=12, b=32, d=1, c=9)))
    print(get_now_seconds())
    print(str_md5('qwe'))
    print(gen_uuid())



# -*- coding: utf-8 -*-
# @Time    : 19/10/12 下午12:13
# @Author  : cc
# @FileName: __init__.py.py
from .middleware_eum import *
from .task_publisher import *
from .task_consumer import *
from .init_config_file import use_config_form_distributed_frame_config_module
from multiprocessing import set_start_method
import platform
if platform.system() == 'Darwin' or platform.system() == 'Linux':
    set_start_method("fork")

use_config_form_distributed_frame_config_module()



# -*- coding:utf-8 -*-
# @Author cc
# @TIME 2020/5/10 00:26
import abc
import os

from leek.utils import get_host_ip, get_day_str


class BaseQueue(metaclass=abc.ABCMeta):
    def __init__(self, queue_name, fliter_rep=False, namespace='', **kwargs):
        """

        :param name: 队列名称
        :param fliter_rep: 是否开始重复任务过滤
        :param namespace: 队列名命名空间
        :param kwargs: db连接动态参数
        """
        if namespace:
            self.queue_name = '%s:%s' % (namespace, queue_name)
        else:
            self.queue_name = queue_name
        self.fliter_rep = fliter_rep
        self.key_sets = self.queue_name + ':filter'

        self.heartbeat_field = f"{self.queue_name}:heartbeat_{os.getpid()}_{get_host_ip()}_{get_day_str()}"
        self.un_ack_sets = f"{self.heartbeat_field}:unack_message"
        self.heartbeat_key = f"tasks:heartbeat:check"
        self._db = self._getconn(**kwargs)

    @abc.abstractmethod
    def _getconn(self, **kwargs):
        pass

    @abc.abstractmethod
    def getdb(self):
        pass

    @abc.abstractmethod
    def qsize(self):
        pass

    @abc.abstractmethod
    def isempty(self):
        pass

    @abc.abstractmethod
    def put(self, item):
        pass

    @abc.abstractmethod
    def clear(self):
        pass

    @abc.abstractmethod
    def get(self, block=False, timeout=None):
        pass

    def check_has_customer(self, hash_value):
        return False

    def add_customer_task(self, hash_value):
        pass

    def ack(self, value):
        pass

    def un_ack(self, value):
        pass

    def dlq_re_consume(self):
        pass



# -*- coding: utf-8 -*-
# @Author  : cc
# @Time    : 2020/6/11 0011 0:56
"""

使用覆盖的方式，做配置。
"""
import sys
import time
import importlib
from pathlib import Path

config_file_content = '''# -*- coding: utf-8 -*-
"""
此文件leek_config.py是自动生成的leek框架的数据库配置文件。
"""
# redis连接配置
#redis_host = '127.0.0.1'
#redis_password = ''
#redis_port = 6379
#redis_db = 0

# kafka配置连接信息
#kafka_port = 9092
#kafka_host = '127.0.0.1'
#kafka_username = ''
#kafka_password = ''
'''


def use_config_form_distributed_frame_config_module():
    """
    自动读取配置,没有则读取项目根目录下的leek_config.py
    :return:
    """
    line = sys._getframe().f_back.f_lineno
    file_name = sys._getframe(1).f_code.co_filename
    try:
        m = importlib.import_module('leek_config')
        msg = f'读取到\n "{m.__file__}:1" 配置文件了\n'
        sys.stdout.write(f'{time.strftime("%H:%M:%S")}  "{file_name}:{line}"   {msg} \n \033[0m')
    except ModuleNotFoundError:
        msg = '在你的项目根目录下生成了 leek_config.py配置文件，快去看看并修改一些自定义配置吧'
        sys.stdout.write(f'{time.strftime("%H:%M:%S")}  "{file_name}:{line}"   {msg} \n \033[0m')
        auto_creat_config_file_to_project_root_path()


def auto_creat_config_file_to_project_root_path():
    if Path(sys.path[1]).as_posix() in Path(__file__).parent.parent.absolute().as_posix():
        pass
    with (Path(sys.path[1]) / Path('leek_config.py')).open(mode='w', encoding='utf8') as f:
        f.write(config_file_content)


if __name__ == '__main__':
    auto_creat_config_file_to_project_root_path()
